@@ -215,9 +215,11 @@ EXPR_EXPR = [
     TokenType.STRING_LITERAL,
     TokenType.IDENTIFIER,
     TokenType.SELECTOR_IDENTIFIER,
+    TokenType.SELECTOR,
+    TokenType.LAMBDA_FUNCTION
 ]
 
-EXPR_OP = list("+-*/%")
+EXPR_OP = list("+-*/%") + ["**"]
 
 
 def is_time(token: Token):
@@ -451,6 +453,7 @@ def chain_tokens_iterate(tokens: List[Token], index: List[int]) -> List[Token]:
         TokenType.FLOAT_LITERAL,
         TokenType.STRING_LITERAL,
         TokenType.FUNCTION_CALL,
+        TokenType.SELECTOR,
     } or (isinstance(t0, GroupToken) and t0.open.value not in {"(", "["}):
         return branch
     while True:
@@ -470,13 +473,15 @@ def chain_tokens_iterate(tokens: List[Token], index: List[int]) -> List[Token]:
                         or tn.type == TokenType.BLOCK_IDENTIFIER
                         or tn.type == TokenType.SELECTOR_IDENTIFIER
                         or tn.type == TokenType.INT_LITERAL
+                        or tn.type == TokenType.FLOAT_LITERAL
+                        or tn.type == TokenType.SELECTOR
                 )
                 and tl.value == "."
                 and tl.end == tn.start
         ):
             branch.append(tn)
             continue
-        if isinstance(tn, GroupToken) and tn.open.value == "[":
+        if isinstance(tn, GroupToken) and tn.open.value in {"[", "("}:
             branch.append(tn)
             continue
         index[0] -= 1
@@ -523,7 +528,15 @@ def make_expr(chains: List[List[Token]]) -> List[List[Token]]:
 
     Input: [Token('a'), Token('+'), Token('b')] or a + b
     Output: [Token('a'), Token('b'), Token('+')] or a b +
+
+    For more information: https://en.wikipedia.org/wiki/Shunting_yard_algorithm
     """
+    new_chains = []
+    for index, chain in enumerate(chains):
+        if chain[0].type in ENDERS:
+            continue
+        new_chains.append(chain)
+    chains = new_chains
     if len(chains) == 0:
         return []
     stack = []
@@ -672,22 +685,21 @@ def parse_iterate(
 
     if t0.value == "import":
         t1 = next_token(tokens, index)
-        if t1 is None or (t1.type != TokenType.IDENTIFIER and t1.type != TokenType.STRING_LITERAL):
-            raise_syntax_error("Expected an identifier or a path", t0)
+        if t1 is None or t1.type != TokenType.STRING_LITERAL:
+            raise_syntax_error("Expected a path for the import statement", t0)
             return False
 
         _as = None
 
-        if t1.type == TokenType.STRING_LITERAL and t1.value[:-1].endswith(".mcfunction"):
-            t2 = next_token(tokens, index)
-            if t2 is None or t2.value != "as":
-                raise_syntax_error("Expected an 'as' keyword", t1)
-
+        t2 = next_token(tokens, index)
+        if t2 and t2.value == "as":
             t3 = next_token(tokens, index)
             if t3 is None or t3.type != TokenType.IDENTIFIER:
                 raise_syntax_error("Expected an identifier", t2)
 
             _as = t3
+        else:
+            index[0] -= 1
 
         statement = ImportStatement(
             code=t0.code, start=t0.start, end=(_as or t1).end, path=t1, as_=_as
@@ -739,12 +751,12 @@ def parse_iterate(
         if not isinstance(body, GroupToken) or body.open.value != "{":
             raise_syntax_error("Expected an expression inside the parentheses", body)
             return False
-        init = parse(spl[0], macros)
+        init = parse(spl[0], macros, class_names)
         if len(init) != 1:
             raise_syntax_error("Expected an expression in the initializer", t0)
         init = init[0]
 
-        it = parse(spl[2], macros)
+        it = parse(spl[2], macros, class_names)
         if len(it) != 1:
             raise_syntax_error("Expected an expression in the iterator", t0)
         it = it[0]
@@ -761,7 +773,7 @@ def parse_iterate(
         )
 
         loop_st_body = [if_st]
-        loop_st_body.extend(parse(body.children, macros))
+        loop_st_body.extend(parse(body.children, macros, class_names))
         loop_st_body.append(it)
 
         loop_st = LoopStatement(
@@ -825,7 +837,7 @@ def parse_iterate(
             start=t0.start,
             end=condition.end,
             time=tim,
-            body=parse(body.children, macros) + [if_st],
+            body=parse(body.children, macros, class_names) + [if_st],
         )
 
         statements.append(loop_st)
@@ -872,7 +884,7 @@ def parse_iterate(
             if_st.body = parse_str("break")[0]
 
         loop_st_body = [if_st]
-        loop_st_body.extend(parse(body.children, macros))
+        loop_st_body.extend(parse(body.children, macros, class_names))
 
         loop_st = LoopStatement(
             code=t0.code,
@@ -906,7 +918,7 @@ def parse_iterate(
             start=t0.start,
             end=body.end,
             time=tim,
-            body=parse(body.children, macros),
+            body=parse(body.children, macros, class_names),
         )
 
         statements.append(statement)
@@ -936,7 +948,7 @@ def parse_iterate(
                 )
             index[0] -= 1
         if isinstance(t2, GroupToken) and t2.open.value == "{":
-            stats = parse(t2.children, macros)
+            stats = parse(t2.children, macros, class_names)
             end_ind = t2.end
         else:
             if t0.value != "else":
@@ -968,6 +980,9 @@ def parse_iterate(
             else_body=None,
         )
         statements.append(statement)
+        if t0.value == "unless":
+            statement.elseBody = stats
+            statement.body = []
         return True
 
     if t0.value == "define":
@@ -1053,14 +1068,17 @@ def parse_iterate(
             start=t0.start,
             end=t2.end,
             name=t1.func if isinstance(t1, GroupToken) and t1.func else t1,
-            body=parse(t2.children, macros),
+            body=parse(t2.children, macros, class_names),
             arguments=[],
             returns=return_type,
         )
         if isinstance(t1, GroupToken) and t1.func:
             spl = split_tokens(t1.children)
             for s in spl:
-                chain = chain_tokens(s)
+                o_chain = chain_tokens(s)
+                chain = o_chain
+                if len(chain) == 3 and chain[0][0].value == "$":
+                    chain = chain[1:]
                 if len(chain) != 2:
                     raise_syntax_error("Expected a valid function argument", s[0])
                 c_type = chain[0]
@@ -1069,7 +1087,7 @@ def parse_iterate(
                     raise_syntax_error(
                         "Expected a valid type for the argument", c_type[0]
                     )
-                statement.arguments.append(chain)
+                statement.arguments.append(o_chain)
 
         statements.append(statement)
         return True
@@ -1105,28 +1123,35 @@ def parse_iterate(
         return True
 
     if t0.value in EXECUTE_MACROS:
-        # as @a at @s ... { <body> }
-        got_tokens = [t0]
-        body_g: GroupToken
-        while True:
-            t = next_token(tokens, index)
-            if t is None:
-                raise_syntax_error("Unexpected end of file", got_tokens[-1])
-                raise SyntaxError("")
-            if isinstance(t, GroupToken) and t.open.value == "{":
-                body_g = t
+        i_start = index[0]
+        is_exec_macro = False
+        for tk in tokens[i_start:]:
+            if isinstance(tk, GroupToken) and tk.open.value == "{":
+                is_exec_macro = True
                 break
-            got_tokens.append(t)
+        if is_exec_macro:
+            # as @a at @s ... { <body> }
+            got_tokens = [t0]
+            body_g: GroupToken
+            while True:
+                t = next_token(tokens, index)
+                if t is None:
+                    raise_syntax_error("Unexpected end of file", got_tokens[-1])
+                    raise SyntaxError("")
+                if isinstance(t, GroupToken) and t.open.value == "{":
+                    body_g = t
+                    break
+                got_tokens.append(t)
 
-        statement = ExecuteMacroStatement(
-            code=t0.code,
-            start=t0.start,
-            end=got_tokens[-1].end,
-            command=" ".join(t.value for t in got_tokens),
-            body=parse(body_g.children, macros),
-        )
-        statements.append(statement)
-        return True
+            statement = ExecuteMacroStatement(
+                code=t0.code,
+                start=t0.start,
+                end=got_tokens[-1].end,
+                command=" ".join(t.value for t in got_tokens),
+                body=parse(body_g.children, macros, class_names),
+            )
+            statements.append(statement)
+            return True
 
     (ind, expr_tokens) = read_expression(tokens, index[0])
     if ind is None:
