@@ -13,7 +13,7 @@ def is_float(s):
         return False
 
 
-OPERATORS = list("+-*/!=%")
+OPERATORS = list("+-/!=%")
 OPERATORS_L = [
     "==",
     "!=",
@@ -24,7 +24,9 @@ OPERATORS_L = [
     "*=",
     "/=",
     "%=",
+    "**=",
     "**",
+    "*",
     "&&",
     "||",
     ">=",
@@ -40,7 +42,7 @@ WORD_OPERATORS = ["is not", "is", "and", "or"]
 INC_OP = ["++", "--"]
 CMP_OP = ["==", "!=", ">", "<", ">=", "<=", "is", "is not"]
 CMP_COMBINE_OP = ["&&", "||", "and", "or"]
-SYMBOL = list("}]){[(:,@\\.~^")
+SYMBOL = list("}]){[(:,@\\.~^$")
 QUOTES = list("\"'")
 WHITESPACE = list(" \t\v")
 EOL_CHAR = "\n"
@@ -98,17 +100,17 @@ from .utils import (
     CplDef,
     CplDefArray,
     CplDefObject,
-    TokenType,
+    TokenType, SELECTOR_TYPE, CplDefFunction,
 )
 
 EmptyToken = Token("", TokenType.SYMBOL, 0, 0)
 
 
 class GroupToken(Token):
-    def __init__(self, code: str, start: int, end: int):
+    def __init__(self, code: str, start: int, end: int, children=None):
         super().__init__(code, TokenType.GROUP, start, end)
         self.parent: GroupToken = EmptyGroup
-        self.children: List[Token | GroupToken] = []
+        self.children: List[Token | GroupToken] = children or []
         self.open: Token = EmptyToken
         self.close: Token = EmptyToken
         self.func: Token | None = None
@@ -150,6 +152,20 @@ class BlockIdentifierToken(Token):
         self.name = name
 
 
+class LambdaFunctionToken(Token):
+    def __init__(
+            self,
+            code: str,
+            start: int,
+            end: int,
+            arguments: GroupToken,
+            body: GroupToken
+    ):
+        super().__init__(code, TokenType.LAMBDA_FUNCTION, start, end)
+        self.arguments = arguments
+        self.body = body
+
+
 def cpl_def_from_tokens(classes, tokens):
     # type: (Dict[str, Any] | List[str], List[Token]) -> CplDef | None
     """
@@ -171,6 +187,17 @@ def cpl_def_from_tokens(classes, tokens):
         type = FLOAT_TYPE
     if t0.value == "string":
         type = STRING_TYPE
+    if t0.value == "selector":
+        type = SELECTOR_TYPE
+    if isinstance(t0, GroupToken) and t0.open.value == "(" and len(t0.children) > 2:
+        t0c = t0.children
+        if not isinstance(t0c[0], GroupToken) or t0c[0].open.value != "(" or t0c[1].value != "=" or t0c[2].value != ">":
+            return None
+
+        type = CplDefFunction(
+            list(map(lambda x: cpl_def_from_tokens(classes, x), split_tokens(t0c[0].children))),
+            cpl_def_from_tokens(classes, t0c[3:])
+        )
     if isinstance(t0, GroupToken) and t0.open.value == "{":
         dct = {}
         spl = split_tokens(t0.children)
@@ -254,7 +281,7 @@ def group_tokens(tokens: List[Token]) -> List[Token]:
                     )
                     and len(pc[-1].selector.value
                             if isinstance(pc[-1], SelectorIdentifierToken)
-                            else "") == 2  # has to be a short selector like @a, @r
+                            else pc[-1].value) == 2  # has to be a short selector like @a, @r
             )
 
             # func(arguments...)
@@ -308,11 +335,28 @@ def group_tokens(tokens: List[Token]) -> List[Token]:
             parent.end = token.end
             parent.update_value()
 
+            ppc = parent.parent.children
+
+            if (token.value == "}" and len(ppc) > 3 and ppc[-2].value == ">" and ppc[-3].value == "="
+                    and ppc[-4].type == TokenType.GROUP and ppc[-4].open.value == "("):
+                ppc.pop()
+                ppc.pop()
+                ppc.pop()
+                arguments = ppc.pop()
+                ppc.append(
+                    LambdaFunctionToken(
+                        parent.code,
+                        arguments.start,
+                        parent.end,
+                        arguments,
+                        parent
+                    )
+                )
+
             if parent._temp == "block":
-                ppc = parent.parent.children
                 colon = ppc.pop()
                 ident = ppc.pop()
-                parent.parent.children.append(
+                ppc.append(
                     BlockIdentifierToken(
                         parent.code,
                         ident.start,
@@ -323,12 +367,11 @@ def group_tokens(tokens: List[Token]) -> List[Token]:
                     )
                 )
 
-            merge = None
             if (
                     parent.type == TokenType.SELECTOR
                     or parent.type == TokenType.SELECTOR_IDENTIFIER
             ):
-                merge = parent.parent.children[-1]
+                merge = ppc[-1]
                 merge.end = parent.end
                 merge.update_value()
                 if isinstance(merge, SelectorIdentifierToken):
@@ -367,6 +410,7 @@ def _tokenize_iterate(
         if (
                 len(tokens) > 1
                 and tokens[-1].value == ":"
+                and tokens[-1].end == index[0]
                 and tokens[-2].type == TokenType.IDENTIFIER
         ):
             token = SelectorIdentifierToken(
@@ -435,7 +479,8 @@ def _tokenize_iterate(
         return
 
     for op in OPERATORS_L:
-        if code[index[0]: index[0] + len(op) + (op in WORD_OPERATORS)] == op:
+        op_got = code[index[0]: index[0] + len(op)]
+        if op_got == op and (op not in WORD_OPERATORS or code[index[0] + len(op)] == " "):
             tokens.append(Token(code, TokenType.OPERATOR, index[0], index[0] + len(op)))
             index[0] += len(op) - 1
             return
@@ -443,7 +488,7 @@ def _tokenize_iterate(
         tokens.append(Token(code, TokenType.OPERATOR, index[0], index[0] + 1))
         return
     if char in QUOTES:
-        startIndex = index[0]
+        start_index = index[0]
         index[0] += 1
         backslash = False
         while index[0] < len(code):
@@ -457,13 +502,13 @@ def _tokenize_iterate(
             index[0] += 1
         if index[0] == len(code) or code[index[0]] == EOL_CHAR:
             raise_syntax_error_t(
-                "Unterminated string", code, startIndex, startIndex + 1
+                "Unterminated string", code, start_index, start_index + 1
             )
-        value = code[startIndex + 1: index[0]]
-        token: Token = Token(code, TokenType.STRING_LITERAL, startIndex, index[0] + 1)
+        value = code[start_index + 1: index[0]]
+        token: Token = Token(code, TokenType.STRING_LITERAL, start_index, index[0] + 1)
         tokens.append(token)
         return
-    startIndex = index[0]
+    start_index = index[0]
     index[0] += 1
     while index[0] < len(code):
         char2 = code[index[0]]
@@ -473,7 +518,7 @@ def _tokenize_iterate(
         index[0] += 1
     if index[0] == len(code):
         index[0] -= 1
-    value = code[startIndex: index[0] + 1]
+    value = code[start_index: index[0] + 1]
     type = TokenType.IDENTIFIER
     if value in KEYWORDS:
         type = TokenType.KEYWORD
@@ -482,24 +527,26 @@ def _tokenize_iterate(
     if value not in KEYWORDS and len(tokens) > 0 and tokens[-1].value == "-" and (
             len(tokens) == 1 or tokens[-2].type in {TokenType.OPERATOR, TokenType.SYMBOL}):
         tokens.pop()
-        startIndex -= 1
+        start_index -= 1
     if (
             type == TokenType.INT_LITERAL
             and len(tokens) > 0
             and tokens[-1].value == "."
-            and tokens[-1].end == startIndex
-            and (
-            len(tokens) == 1
-            or tokens[-2].end != startIndex - 1
-            or tokens[-2].type == TokenType.INT_LITERAL
-    )
-    ):
+            and tokens[-1].end == start_index
+            and (len(tokens) == 1 or tokens[-2].end != start_index - 1 or tokens[-2].type == TokenType.INT_LITERAL)):
         if len(tokens) > 1 and tokens[-2].type == TokenType.INT_LITERAL:
             tokens.pop()
         start = tokens.pop().start
         tokens.append(Token(code, TokenType.FLOAT_LITERAL, start, index[0] + 1))
         return True
-    tokens.append(Token(code, type, startIndex, index[0] + 1))
+    new_t = Token(code, type, start_index, index[0] + 1)
+    if type == TokenType.IDENTIFIER and len(tokens) > 1 and tokens[-1].value == ":" and tokens[
+        -2].type == TokenType.IDENTIFIER:
+        sep = tokens.pop()
+        ident = tokens.pop()
+        tokens.append(SelectorIdentifierToken(code, ident.start, index[0] + 1, new_t, sep, ident))
+        return True
+    tokens.append(new_t)
 
 
 def _tokenize(code: str, can_macro=True):
